/*	Copyright  (c)	Günter Woigk 2001 - 2014
  					mailto:kio@little-bat.de

	This file is free software

 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	• Redistributions of source code must retain the above copyright notice,
	  this list of conditions and the following disclaimer.
	• Redistributions in binary form must reproduce the above copyright notice,
	  this list of conditions and the following disclaimer in the documentation
	  and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
	PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
	CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
	EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
	ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef file_utilities_h
#define	file_utilities_h

#include "kio/kio.h"
#include "MyFileInfo.h"

#ifdef  HAVE_SYS_DIRENT_H
#include <sys/dirent.h>
#endif




//#include "file_utilities.h"
//#include "s_type.h"
#include "kio/kio.h"


extern	s_type	classifyFile	( FILE* );
inline	bool	isFile			( FILE* file )					{ return classifyFile(file)==s_file; }
inline	bool	isDir			( FILE* file )					{ return classifyFile(file)==s_dir; }
inline	bool	isTTY			( FILE* file )					{ return classifyFile(file)==s_tty; }
extern	void	setBlocking		( FILE*, bool onoff );
extern	void	setAsync		( FILE*, bool onoff );
extern	off_t	fileLength		( FILE* );						// returns -1 and sets errno on error

extern	str 	newReadStr		( FILE* ) throw(bad_alloc);		// read from file up to EOF,0,10,13,10+13,13+10




/* universal methods:
*/
inline	s_type	classifyFile	( mode_t mode )							{ return s_type(mode>>12); }
extern	str		modeStr			( mode_t );
extern	str		effModeStr		( mode_t, gid_t, uid_t );
extern	cstr	latinStr1   	( cstr non_utf8_str );					// #define USE_LATIN1_PATH

extern	cstr	filenameFromPath		( cstr path );
extern	cstr	extensionFromPath		( cstr path );
inline	str		extensionFromPath		( str  path )					{ return (str)extensionFromPath( (cstr)path ); }
extern	cstr	basenameFromPath		( cstr path );
extern	cstr	directoryFromPath		( cstr path );
extern	cstr	lastComponentFromPath	( cstr path );

extern	void	changeWorkingDir( cstr path );
extern	cstr	workingDir		( );
extern	cstr	homeDir			( );
extern	cstr	tempFileName	( cstr same_dir_as_file );				// renameable to 'file'
extern	cstr	quickFullPath	( cstr path );							// just add workingdir or homedir


// __________________________________________________________________
// PATH methods:
//
extern	s_type	classifyFile	( cstr path, bool resolve_last_symlink=1 );
inline	bool	isFile			( cstr path, bool resolve_last_symlink=1 )	{ return classifyFile(path,resolve_last_symlink)==s_file; }
inline	bool	isDir			( cstr path, bool resolve_last_symlink=1 )	{ return classifyFile(path,resolve_last_symlink)==s_dir; }
inline	bool	isLink			( cstr path )								{ return classifyFile(path,no)==s_link; }
inline	bool	isTTY			( cstr path )								{ return classifyFile(path)==s_tty; }

extern	time_t	fileMTime		( cstr path, bool resolve_last_symlink=1 );		// modification time
extern	time_t	fileATime		( cstr path, bool resolve_last_symlink=1 );		// last access time
extern	time_t	fileCTime		( cstr path, bool resolve_last_symlink=1 );		// last status change time

extern	off_t	fileLength		( cstr path, bool resolve_last_symlink=1 );		// returns -1 and sets errno on error

extern	str		fullPathToNode	(  str path, bool resolve_last_symlink=1, bool auto_create_path=no );
extern	str		fullPathToNode	( cstr path, bool resolve_last_symlink=1, bool auto_create_path=no );
extern	cstr	fullPath		( cstr path, bool resolve_last_symlink   );		// returns cstr in strPool

//extern void	renameNode		( cstr oldname, cstr newname, bool overwrite, bool resolve_last_symlink, s_type typ=s_any );
extern	void	renameNode		( cstr oldpath, cstr newpath, bool overwrite );
extern	void	deleteNode		( cstr path, bool resolve_last_symlink, s_type typ=s_any );
extern	void	deleteDir		( cstr dpath, bool fulltree=no );
extern	void	swapFiles		( cstr path1, cstr path2 );

extern	void	createFile		( cstr path, int8 const* data, uint32 len, int how='w'/*overwrite*/, int mode=0660 );



// __________________________________________________________________
// FD methods:
//

extern	s_type	classifyFile	( int fd );

extern	void	setBlocking		( int fd, bool onoff );
extern	void	setAsync		( int fd, bool onoff );

extern	time_t	fileMTime		( int fd );			// modification time
extern	time_t	fileATime		( int fd );			// last access time
extern	time_t	fileCTime		( int fd );			// last status change time

extern	off_t	fileLength		( int fd );			// returns -1 and sets errno on error

extern	int		newTempFile		();					// create & open temp file in /tmp/

extern	int		getTerminalSize	( int tty, int& rows, int& cols );
extern	int		getTerminalRows	( int tty );
extern	int		getTerminalCols	( int tty );
extern	int		getTerminalWidth( int tty );		// pixel
extern	int		getTerminalHeight(int tty );		// pixel
extern	int		setTerminalSize	( int tty, int rows, int cols );		// does not work !?



#endif


